import random
import time
import sys
from . import utils as u
from . import beings as b
from . import items as i
from . import event as e
from . import rooms as r
from . import commands as c
# This is the main file for the game. It contains the main game loop and the
# main game logic. It also contains the room and item classes, as well as the
# commands and the player class.


def show_instructions():
    # Print a main menu and the commands
    print(
        """
        RPG Game
        ========

        Get to the Garden with the key
        Avoid the monsters!
        """
    )
    c.show_commands()


def show_status():
    # Print the player's current status
    print('---------------------------')
    print('Health: ' + str(player.health))
    print('Attack: ' + str(player.attack_power))
    print('Defence: ' + str(player.defence))
    print('XP: ' + str(player.XP))
    print('Level: ' + str(player.level))
    # Print the current inventory
    print('Inventory: ' + str(player.inventory))
    print(r.directions(player.current_room))
    print(r.rooms[player.current_room].info)
    print("---------------------------")


# A player, which initially has an empty inventory
player = b.Player(
    current_room = 'Hall',
    last_room = 'Hall',
    health = 10,
    attack = 3,
    defence = 1,
    name = 'Player',
    inventory = i.Inventory(),
    XP=0,
    LVL=1,
)

def move(*directions):
    global player
    for direction in directions:
        if (r.rooms[player.current_room].directions and
            direction in r.rooms[player.current_room].directions):
            player.last_room = player.current_room
            player.current_room = r.rooms[player.current_room].directions[direction]
        elif u.DEVMODE and direction in r.rooms:
            player.last_room = player.current_room
            player.current_room = direction
        else:
            print(f'You can\'t go {direction}!')

def get(*item_names):
    global player
    # If the room contains an item, and the item is the one they want to get
    for item_name in item_names:
        # Find the item in the room by its name
        item = next((i for i in r.rooms[player.current_room].items if i.name == item_name), None)
        if item:
            # Add the item to their inventory
            if player.inventory.add(item):
                # Remove the item from the room
                r.rooms[player.current_room].items.remove(item)
            else:
                continue
        else:
            # Tell them they can't get it
            print(f'Can\'t get {item_name}!')

def quit(time_to_wait: int=0):
    try:
        if type(time_to_wait) == list:
            time_to_wait = time_to_wait[0]
        if type(time_to_wait) == str:
            time_to_wait = int(time_to_wait)
    except ValueError:
        print('Invalid time to wait!')
        sys.exit()
    print('Quitting...')
    time.sleep(time_to_wait)
    sys.exit()

def drop(*item_names):
    global player
    # If the players inventory contains an item, the player can drom that item
    for item_name in item_names:
        # Find the item in the inventory by its name
        item = next((i for i in player.inventory.items if i.name == item_name), None)
        if item:
            # Add the item to the room
            r.rooms[player.current_room].i.append(item)
            # Remove the item from their inventory
            u.DEBUG(player.inventory.remove(item))
            print(f'Dropped {item_name}!')
        else:
            # Tell them they can't drop it
            print(f'Can\'t drop {item_name}!')

def look(*Args):
    print('You see:')
    # Print the items in the room
    if type(r.rooms[player.current_room]) == r.Room:
        if r.rooms[player.current_room].items:
            print(', '.join(i.name for i in r.rooms[player.current_room].items))
        else:
            print('Nothing!')


def battle(player:b.Player, monster:b.Monster):
    print(f'A {monster.type} attacks you!')
    print(f'Enemy health: {monster.health}')
    print(f'Your health: {player.health}')
    while player.health > 0 and monster.health > 0:
        input_ = input('Do you want to run or attack? ').strip().lower()
        if input_ == 'run':
            player.current_room = player.last_room
            print('You ran away!')
            return
        elif input_ == 'attack':
            
            player.attack(monster)
            if monster.health > 0:
                monster.attack(player)
                print(f'Enemy health: {monster.health}')
                print(f'Your health: {player.health}')
            else:
                print(f'You killed the {monster.type}!')
                print(f'Your health: {player.health}')
                return
        else:
            print('Invalid command!')
    if player.health <= 0:
        player.die()
        return

def handle_monster(room: r.Room):
    global player
    monster = room.monster
    if type(monster) == b.Monster:
        if monster.health > 0:
            if (monster.needs and
                all(player.inventory.has(x)
                for x in monster.needs)):
                if monster.killed_text:
                    print(monster.killed_text)
                elif monster.type:
                    print(
                        'You killed the ' +
                        monster.type +
                        '!'
                    )
                else:
                    print('You killed the monster!')
                room.monster = None
            else:
                battle(player, monster)

c.commands = {
    'go': {
        "command": move,
        "description": "Move to a different room",
        "args": "one or more directions",
    },
    'get': {
        "command": get,
        "description": "Get item items from the room",
        "args": "one or more items",
    },
    'quit': {
        "command": quit,
        "description": "Quit the game",
        "args": "seconds to wait before quitting",
    },
    'help': {
        "command": c.show_commands,
        "description": "Show the commands",
        "args": "number of commands to show. 0 to show all",
    },
    'drop': {
        "command": drop,
        "description": "Drop an item into the current room",
        "args": "one or more items",
    },
    'look': {
        "command": look,
        "description": "Look around the room",
        "args": "anything",
    },
}


def main():
    # Show the instructions
    print("Loading...")
    time.sleep(0.1)
    print("Setting up player...")
    time.sleep(0.1)
    print("Setting up command functions...")
    time.sleep(0.1)
    print("Setting up commands dictionary...")
    time.sleep(0.1)
    print("Finishing up...")
    time.sleep(0.1)
    print("Done!\n\n")
    show_instructions()

    # Loop forever
    while True:

        show_status()

        # Get the player's next 'move'
        # parse_command() breaks it up into a list array
        # eg typing 'go east' would give the list:
        # ['go', 'east'] because go is a commannd and east is the direction,
        # but if a multiword command is added to the game, it will show
        # something like this (the bit where it says 'multiword command'
        # is where the multiword command would be, and the bit where it says
        # 'argument(s)' is where the argument(s) would be):
        # ['multiword command', 'argument(s)']
        command = ''
        while command == '':
            command = input('>')

        command = c.parse_command(command)

        room = r.rooms[player.current_room]
        if type(room) == r.Room:
            if (room.exit and
                room.exit.needs):
                if all(player.inventory.has(x) 
                    for x in room.exit.needs):
                    print('You escaped the house... YOU WIN!')
                    sys.exit()
                else:
                    print(
                        'You need ' +
                        ', '.join(r.rooms[player.current_room].exit.needs) +
                        ' to escape!'
                    )

            if command and len(command) >= 2:
                c.commands[command[0]]["command"](*command[1:])
            else:
                print('Invalid command!')

            room = r.rooms[player.current_room]

            if room.monster:
                handle_monster(room)

if __name__ == '__main__':
    u.handle_devmode()
    main()



import time
import sys
import os
import random
# This file contains the utility functions for the game.
# It contains the functions for the game loop, the commands, and the player class.

global DEVMODE, DEVPIN
DEVMODE = False
DEVPIN = 'alexander793510'

def DEBUG(*args):
    if DEVMODE:
        print('DEBUG:', *args)


def rickroll(times=1):
    print("you hear a strange noise...")
    time.sleep(1)
    for _ in range(times):
        print("Never gonna give you up,")
        time.sleep(1)
        print("Never gonna let you down,")
        time.sleep(1)
        print("Never gonna run around and desert you.")
        time.sleep(1)
        print("Never gonna make you cry,")
        time.sleep(1)
        print("Never gonna say goodbye,")
        time.sleep(1)
        print("Never gonna tell a lie and hurt you.")
        time.sleep(1)
    return


def handle_devmode():
    devmode = input('Do you want to run in devmode? yes/no: ').strip().lower().startswith('y')
    if devmode:
        pin = input("Enter the pin to enter devmode: ").strip()
        if pin == DEVPIN:
            print("Entering devmode...")
            DEVMODE = True
        else:
            print("Incorrect pin.")
            DEVMODE = False
    else:
        DEVMODE = False


from . import utils as u

global commands
commands = {}

def show_commands(commands_to_show=0):
    global commands
    try:
        if type(commands_to_show) == list:
            commands_to_show = commands_to_show[0]
        if type(commands_to_show) == str:
            commands_to_show = int(commands_to_show)
    except ValueError:
        print('invalid number of commands to show!')
        commands_to_show = 0
    # Print a list of the commands
    print(
        """
        Commands:
        """
    )
    if commands_to_show == 0:
        commands_to_show = len(commands)
    shown_commands = 0
    for key, value in commands.items():
        print(f'        {key} - {value["description"]} - {value["args"]}')
        shown_commands += 1
        if shown_commands >= commands_to_show:
            return


# A function to split an innput str into a list, with the first entry being
# the command, which may be multiple words
def parse_command(command: str) -> list:
    global commands
    for key in commands.keys():
        u.DEBUG('Checking command "' + command + '" with key "' + key + '"')
        if command.startswith(key):
            if len(command) == len(key):
                return [key.strip()]
            return [key.strip()] + command[len(key):].strip().split()
    return False




from . import utils as u
from . import items as i
# This file contains the event class and the events for the game.

class Event:
    def __init__(self, chance, code:lambda:None):
        self.chance = chance
        self.code = code


from . import utils as u
from . import event as e
from . import items as i
from . import beings as b
# This file contains the room class and the rooms for the game.

class Exit:
    def __init__(self, exit_type, needs=None):
        self.type = exit_type
        self.needs = needs or []

class Room:
    def __init__(
            self,
            directions=None,
            items:list[i.Item]=None,
            monster=None,
            info='',
            exit:Exit=None,
            events:list[e.Event]=[],
            rtype:str='house',
        ):
        self.directions = directions or {}
        self.items = items or []
        self.monster = monster
        self.info = info
        self.exit = exit
        self.events = events
        self.type = rtype

roomDirectionText = {
    'house': {
        'north': 'There is a door to the north',
        'south': 'There is a door to the south',
        'east': 'There is a door to the east',
        'west': 'There is a door to the west',
        'up': 'There is a staircase going up',
        'down': 'There is a staircase going down',
    },
}

def directions(currentRoom):
    text = ''
    if currentRoom in rooms:
        for direction in rooms[currentRoom].directions:
            if direction in roomDirectionText[rooms[currentRoom].type]:
                text += roomDirectionText[rooms[currentRoom].type][direction] + '\n'
    return text

# A dictionary linking a room to other rooms
rooms = {
    'Hall': Room(
        directions = {
            'south': 'Kitchen',
            'east': 'Dining Room',
        },
        items = [
            i.Item('key'),
        ],
        info = 'You are in a dark, dusty hall.',
    ),
    'Kitchen': Room(
        directions = {
            'north': 'Hall',
            'east': 'Garden',
        },
        monster = b.Monster(
            m_type = 'bear',
            health = 7,
            attack = 4,
            defence = 2,
            needs = [
                'potion',
            ],
            kill_text = 'The bear eats you.',
            killed_text = 'You killed the bear with a potion!',
        ),
        info = 'You are in a large kitchen.',
    ),
    'Dining Room': Room(
        directions = {
            'west': 'Hall',
            'south': 'Garden',
        },
        items = [
            i.Item('potion'),
        ],
        info = 'You are in a large dining room.',
    ),
    'Garden': Room(
        directions = {
            'north': 'Dining Room',
            'west': 'Kitchen',
        },
        info = 'You are in a beautiful garden, with a locked gate leading \
out of the garden.',
        exit = Exit(
            exit_type = 'gate',
            needs = [
                'key'
            ],
        ),
    )
}


import random
import sys
from . import items
from . import utils as u
from . import event as e
# This file contains the classes for the beings in the game.
# The Player class and the Monster class are subclasses of the Being class.

class Being:
    def __init__(self, health, attack, defence, name):
        self.health = health
        self.attack_power = attack
        self.defence = defence
        self.name = name

    def calculate_damage(self, target):
        if issubclass(type(target), Being):
            max_damage = self.attack_power + 2
            min_damage = max_damage // 2
            base_damage = random.randint(min_damage, max_damage)
            damage = base_damage - target.defence
            if damage < 0:
                damage = 0
            return damage

    def attack(self, target):
        if issubclass(type(target), Being):
            end_damage = self.calculate_damage(target)
            target.health -= end_damage
            print(
                str(self.name) +
                ' attacked ' +
                str(target.name) +
                ' for ' +
                str(end_damage) +
                ' damage! ' +
                str(target.name) +
                ' had ' +
                str(target.defence) +
                ' defence! '
            )
            if target.health <= 0:
                target.health = 0
                target.die()

    def die(self):
        print(f'{self.name} died!')
        if type(self) == Player:
            print('Game Over!')
            sys.exit()

class Monster(Being):
    def __init__(
            self,
            m_type,
            health,
            attack,
            defence,
            needs=None,
            kill_text='',
            killed_text='',
            destroy_needs=True,
        ):
        super().__init__(health, attack, defence, m_type)
        self.type = m_type
        self.needs = needs or []
        self.kill_text = kill_text
        self.killed_text = killed_text
        self.destroy_needs = destroy_needs

class Player(Being):
    def __init__(
            self,
            current_room,
            last_room,
            health,
            attack,
            defence,
            name,
            inventory:items.Inventory = items.Inventory(),
            XP=0,
            LVL=1
        ):
        super().__init__(health, attack, defence, name)
        self.current_room = current_room
        self.last_room = last_room
        self.inventory = inventory
        self.XP = XP
        self.level = LVL
        self.deaths = 0
        self.max_health = health
    
    def die(self):
        self.deaths += 1
        print(f'You died! You have died {self.deaths} times!')
        shouldrickroll = input('Do you want to rewind time (for a price)? yes/no: ').strip().lower().startswith('y')
        if shouldrickroll:
            u.rickroll(self.deaths)
            self.health = self.max_health
            self.current_room = self.last_room
        else:
            super().die()



from . import utils as u

class Item:
    def __init__(self, name, size=1.0):
        self.name = name
        self.size = size


class Inventory:
    def __init__(self):
        self.items = []
        self.size = 10.0

    def add(self, item):
        used_size = sum(item.size for item in self.items)
        if used_size + item.size <= self.size:
            self.items.append(item)
            print(f'{item.name} got!')
            return True
        else:
            print(f'Not enough space to add {item.name}!')
            return False

    def has(self, item):
        return any(i.name == item for i in self.items)
    
    def find(self, item):
        # Finds an item in the inventory and returns the item object
        # or None if not found, and also returns the index of the item in the
        # inventory This is used to remove the item from the inventory
        if self.has(item):
            for i in self.items:
                if i.name == item:
                    return i, self.items.index(i)
        return None, None

    def remove(self, item):
        if type(item) == Item:
            item = item.name
        i, itemplace = self.find(item)
        u.DEBUG(i, itemplace)
        if i:
            self.items.remove(i)
            return True
        return False

    def __str__(self):
        inventory = ', '.join(item.name for item in self.items)
        if inventory == '':
            inventory = 'empty'
        som = sum(item.size for item in self.items)
        used_size = som if som > 0 else 0.0
        inventory += f' ({used_size}/{self.size})'
        return inventory



from setuptools import setup, find_packages

setup(
    name="T_BAG",
    version="1.1.3",  # Update version number for each release
    packages=find_packages(),  # Automatically finds all packages and modules
    install_requires=[],
    entry_points={
        "console_scripts": [
            "T_BAG=T_BAG.main:main",
        ],
    },
    python_requires=">=3.6",  # Specify Python version compatibility
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    description="Text-Based Adventure Game source code.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Flameblade375/Text-based-Adventure-Game-Source-Code",
    author="Alexander.E.F",
    author_email="alexander@xandy.rocks",
    license="MIT",
)
